#include "plots_treeTools.h"

#include <TFile.h>
#include <TString.h>
#include <TTree.h>
#include <TH1.h>
#include <TH2.h>
#include <TLegend.h>
#include <TCut.h>

#include <stdlib.h>
#include <map>

using namespace std;

// globals
map<TString, TTree *> g_myTreeMap;
map<TString, double> g_myTreeMapScale;
map<int, TString> g_myTreeOriginalOrdering;
int g_OriginalOrdering = 0;

map<TString, TObject *> g_myHistosMap;
map<TString, double> g_myHistosCalc;

vector<TString> g_histosOrder;

map<TString, TObject *> GetHistosMap(){return g_myHistosMap;}

int GetNHistos(TString idString){
	map<TString, TObject *>::iterator itr = g_myHistosMap.begin();
	int cntr = 0;
	TString tempS;
	for( ; itr != g_myHistosMap.end() ; itr++){
		//cout << (*itr).second->GetName() << endl;
		tempS = (*itr).second->GetName();
		if(tempS.Contains(idString, TString::kExact))
			cntr++;
	}

	return cntr;
}

TString GetHistoKey(int i, TString idString){
	map<TString, TObject *>::iterator itr = g_myHistosMap.begin();
	int cntr = 0;
	TString tempS;
	for( ; itr != g_myHistosMap.end() ; itr++){
		tempS = (*itr).second->GetName();
		if(cntr++ == i && tempS.Contains(idString, TString::kExact)) return (*itr).first;
	}
	return TString("");
}

TObject * GetHistoPtr(int i, TString idString){
	map<TString, TObject *>::iterator itr = g_myHistosMap.begin();
	int cntr = 0;
	TString tempS;
	for( ; itr != g_myHistosMap.end() ; itr++){
		tempS = (*itr).second->GetName();
		if(cntr++ == i && tempS.Contains(idString, TString::kExact)) return (*itr).second;
	}
	return 0x0;
}


void TreeFillMap(TString n, TTree * p){
	g_myTreeMap[n] = p;
}

TTree * GetTreePtr(TString n){
	return  g_myTreeMap[n];
}

int NOfColons(const char * sc){

	std::string s = sc;
	size_t pos = std::string::npos;
	int dims = 1;

	do{

		pos = s.find(':');
		if(pos == std::string::npos) continue;

		s = s.substr(pos+1);
		dims++;

	} while(pos != std::string::npos);

	return dims;
}

TObject * DrawAll(TString baseDraw, TCut cutsDraw, OnCanvasOptions & ocOpts, 
		ActionFlags afl){//, TVirtualPad * vPad){

	// Check the dimension
	int nDim = NOfColons(baseDraw.Data());
	cout << endl;
	cout << "[INFO] attemp to draw " << nDim << "D histogram with cut:" << endl;
	cout << "       "; cutsDraw.Print();

	TString tempDraw = "";
	TString tempHisto = "";

	/*
	map<int, TString>::iterator itrO = g_myTreeOriginalOrdering.begin();
	for( ; itrO != g_myTreeOriginalOrdering.end() ; itrO++) {
		cout << (*itrO).first << " --> " << (*itrO).second << endl;
	}
	*/

	map<TString, TTree *>::iterator itr = g_myTreeMap.begin();
	for(itr = g_myTreeMap.begin() ; itr != g_myTreeMap.end() ; itr++){

		tempDraw = baseDraw;
		tempDraw += ">>";

		// histo name ///////////////////////////////
		tempHisto = "h_";
		// get rid of ":" in the name
		TString baseReplacement = baseDraw;
		baseReplacement.ReplaceAll(':', "_");
		baseReplacement.ReplaceAll('[', "");
		baseReplacement.ReplaceAll(']', "");
		baseReplacement.ReplaceAll('$', "");
		baseReplacement.ReplaceAll('(', "");
		baseReplacement.ReplaceAll(')', "");
		tempHisto += baseReplacement;
		tempHisto += "_";
		tempHisto += (*itr).first;
		/////////////////////////////////////////////

		tempDraw += tempHisto;
		// Add constructor parameters here
		tempDraw += ocOpts.constructorStr;

		// Draw
		(*itr).second->Draw(tempDraw, cutsDraw);
		// save histos pointers
		// do not use gPad !
		//g_myHistosMap[(*itr).first] = gPad->GetPrimitive(tempHisto);
		g_myHistosMap[(*itr).first] = gDirectory->Get(tempHisto);
		// Scale to the factor in GetTree(...)

		((TH1 *)(g_myHistosMap[(*itr).first]))->Scale(g_myTreeMapScale[(*itr).first]);


		// histogram integrals to calculate if requested
		g_myHistosCalc[(*itr).first] = 0.;

		if(afl.doIntegral){
			// do nothing for now
		}

		cout << "[DRAW] "
				<< tempDraw << " --> "
				<< tempHisto << " ("
				<< g_myHistosMap[(*itr).first]
				                 << ") " << (*itr).second
				                 << endl;

		//g_myHistosMap[(*itr).first]->Dump();
	}

	TH1 * tempHistoPtr = 0x0;
	TH2 * tempHisto2DPtr = 0x0;
	if(nDim == 1) tempHistoPtr = (TH1 *) (*(g_myHistosMap.begin())).second;
	if(nDim == 2) tempHisto2DPtr = (TH2 *) (*(g_myHistosMap.begin())).second;
	if(!tempHistoPtr && !tempHisto2DPtr) {
		cout << "[ERROR] no histo at all ?!" << endl;
		exit(1);
	}

	if(nDim == 1 && g_histosOrder.empty() && ocOpts.orderHistos) { // if 1D histos and if order is empty

		map<TString, TObject *>::iterator itr2;

		// decide the order, highest first
		for(itr2 = g_myHistosMap.begin() ; itr2 != g_myHistosMap.end() ; itr2++)
			g_histosOrder.push_back( (*itr2).first );

		cout << "[INFO] ordering ..." << endl;

		double maxh = 0.;
		maxh = tempHistoPtr->GetMaximum();

		TString temph = "";
		// start comparing from the second histo

		int cntr = 1;
		itr2 = g_myHistosMap.begin();
		itr2++;
		for( ; itr2 != g_myHistosMap.end() ; itr2++)
		{
			tempHistoPtr = static_cast<TH1 *>((*itr2).second); // works for TH2 as well

			if(tempHistoPtr->GetMaximum() > maxh){

				maxh = tempHistoPtr->GetMaximum();

				temph = g_histosOrder[cntr];
				g_histosOrder[cntr] = g_histosOrder[cntr-1];
				g_histosOrder[cntr-1] = temph;
			}
			cntr++;
		}
	}

	// if no order required, use the original ordering

	if( nDim == 1 && g_histosOrder.empty() && !ocOpts.orderHistos ) {

		for(int itr2 = 0 ; itr2 < (int)g_myTreeOriginalOrdering.size() ; itr2++) {
			//cout << " --> " << g_myTreeOriginalOrdering[itr2] << endl;
			g_histosOrder.push_back( g_myTreeOriginalOrdering[itr2] );
		}

	}

	// print order and draw !
	vector<TString>::iterator itr3 =  g_histosOrder.begin();
	cout << "       Order : ";
	int linecolor = 1;
	TString drawOpt = ocOpts.drawOpt; // initialize to user-defined

	//vPad->cd();

	pair<TH1 *, double> storeMax;
	storeMax.first = 0x0;
	storeMax.second = 0;
	double minims = -1;
	for( ; itr3 != g_histosOrder.end() ; itr3++){
		cout << *itr3;
		if(itr3+1 != g_histosOrder.end() ) cout << " , ";
		else cout << endl;
		// draw
		if(nDim == 1) {
			tempHistoPtr = dynamic_cast<TH1 *>(g_myHistosMap[*itr3]);
			tempHistoPtr->SetLineColor(linecolor++);

			if(!ocOpts.drawNormalized) tempHistoPtr->Draw(drawOpt);
			else tempHistoPtr = tempHistoPtr->DrawNormalized(drawOpt);

			//tempHistoPtr->Fit("gaus","","same",0.05,1.2);

			//tempHistoPtr->GetXaxis()->SetRangeUser(0,10);
			// the order has been decided in the first plot
			// but in a second iteration we may need to
			// RangeUser
			if(tempHistoPtr->GetMaximum() > storeMax.second) {

				if(storeMax.first == 0x0) storeMax.first = tempHistoPtr; // store the first pointer only --> contains the right axis
				storeMax.second = tempHistoPtr->GetMaximum();

				if(minims < 0) minims = tempHistoPtr->GetMinimum();

			}

			if(tempHistoPtr->GetMinimum() < minims)
				minims = tempHistoPtr->GetMinimum();

		}else if(nDim == 2){
			tempHisto2DPtr = dynamic_cast<TH2 *>(g_myHistosMap[*itr3]);
			tempHisto2DPtr->SetMarkerColor(linecolor++);
			tempHisto2DPtr->SetMarkerStyle(ocOpts.markerStyle);
			tempHisto2DPtr->Draw(drawOpt);
		}

		// the order has been decided in the first plot
		// but in a second iteration we may need to
		// RangeUser


		drawOpt = "same";
	}

	// Set Range user
	if(nDim == 1) {
		if(ocOpts.setLogY){
			if(ocOpts.drawNormalized && minims == 0)  minims = 1E-6;
			storeMax.first->GetYaxis()->SetRangeUser(minims,  storeMax.second*10.);
		}else{
			storeMax.first->GetYaxis()->SetRangeUser(minims,  storeMax.second*1.1);
		}
		cout << "RangeUser - -> " << storeMax.first << " to " << storeMax.second << endl;
	}

	//if(ocOpts.setLogY) gPad->SetLogy();

	// do legend
	if(ocOpts.doLegend){
		TLegend * t1 = new TLegend(0.6, 0.5, 0.9, 0.9);
		t1->SetBorderSize(1);
		t1->SetFillColor(kWhite);
		for(itr3 =  g_histosOrder.begin() ;
				itr3 != g_histosOrder.end() ;
				itr3++){
			if(nDim == 1) t1->AddEntry(dynamic_cast<TH1 *>(g_myHistosMap[*itr3]), *itr3, "L");
			if(nDim == 2) t1->AddEntry(dynamic_cast<TH2 *>(g_myHistosMap[*itr3]), *itr3, "M");
		}
		t1->Draw();
	}

	// return the first drawn
	return g_myHistosMap[g_histosOrder[0]];
}

void GetTree(TTree * T, TString nickName, double scale){

	g_myTreeMap[nickName] = T;
	g_myTreeMapScale[nickName] = scale;

}

void GetTree(TString filename, TString treeName, TString nickName, double scale){

	TFile * f = new TFile(filename);
	if(!f->IsOpen()){
		cout << "[ERROR] couldn't open file " << filename << endl;
		exit(1);
	}

	TTree * T = (TTree *) f->Get(treeName);

	if(!T){
		cout << "[ERROR] couldn't find tree " << treeName << endl;
		exit(1);
	}
	cout << "Pushing back : " << nickName << endl;
	g_myTreeMap[nickName] = T;
	g_myTreeMapScale[nickName] = scale;
	g_myTreeOriginalOrdering[g_OriginalOrdering++] = nickName;
	//return T;
}

///////////////////////////////////////////////
// OnCanvasOptions
/*
ClassImp(OnCanvasOptions)

OnCanvasOptions::OnCanvasOptions(){

}
 */
